/* LCD Library (for HD44780, C version)
 ****************************************************************************
 * Author:      Hendrik Hlscher                         					*
 * modified by:    															*
 * Revision:    1.0                              							*
 * Date:        19.06.06                  									*
 * Target:      Transceiver Rev.3.01 (www.hoelscher-hi.de)        			*
 * Compiled using AVRStudio4                 								*
 ****************************************************************************
 * Fuses:    OSC:  8.0 Mhz xtal	          									*
 *           WDT:  OFF                       								*
 ****************************************************************************
 *                   Software License Agreement                       		*												*
 * 					                              							*
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,   		*
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED   		*
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 	      		*
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. HENNE'S SITES SHALL NOT,   	*
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 	      		*
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.	              		*
 ****************************************************************************
 */

#include "globals.h"
#include "lib_lcd.h"

#include <stdlib.h>
#include <string.h>

		int8_t  enc_delta;
static  int8_t 	enc_last;				//buffer of encoder state
        uint8_t BtnCnt;

const char Msg[][16] PROGMEM=  {
	{' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '},		//0
	{' ', ' ', 'A', 'n', 'a', 'l', 'y', 'z', 'e', 'r', ' ', '2', '.', '0', ' ', ' '},		//1
	{'h', 'o', 'e', 'l', 's', 'c', 'h', 'e', 'r', '-', 'h', 'i', '.', 'd', 'e', ' '},		//2
	{'>', 'D', 'M', 'X', ' ', 'S', 't', 'a', 't', 'i', 's', 't', 'i', 'c', 's', ' '},		//  3
	{' ', 'D', 'M', 'X', ' ', 'O', 'v', 'e', 'r', 'v', 'i', 'e', 'w', ' ', ' ', ' '},		//  4
	{' ', 'D', 'M', 'X', ' ', 'R', 'e', 'c', 'e', 'i', 'v', 'e', ' ', ' ', ' ', ' '},		//  5	
	{' ', 'D', 'M', 'X', ' ', 'T', 'r', 'a', 'n', 's', 'm', 'i', 't', ' ', ' ', ' '},		//  6
	{' ', 'R', 'D', 'M', ' ', 'C', 'o', 'n', 't', 'r', 'o', 'l', ' ', ' ', ' ', ' '},		//  7
	{' ', 'C', 'h', 'a', 'n', 'n', 'e', 'l', 's', ':', ' ', ' ', ' ', ' ', ' ', ' '},		//    8
	{' ', 'R', 'a', 't', 'e', ':', ' ', ' ', ' ', ' ', 'H', 'z', ' ', ' ', ' ', ' '},		//    9
	{' ', 's', 'e', 'a', 'r', 'c', 'h', 'i', 'n', 'g', '.', '.', '.', ' ', ' ', ' '},		//    10
	{' ', 'n', 'o', ' ', 'R', 'D', 'M', '-', 'D', 'e', 'v', 'i', 'c', 'e', 's', ' '},		//    11
	{' ', 'P', 'e', 'r', 's', 'o', 'n', 'a', 'l', 'i', 't', 'y', ':', ' ', ' ', ' '},		//    12
	};

void enable_lcd(void)
{
asm volatile ("nop");
asm volatile ("nop");
LCDPORT |= (1<<LCD_E);
_delay_us(5);
LCDPORT &= ~(1<<LCD_E);
asm volatile ("nop");
asm volatile ("nop");
}

void write_lcd(char Data,char CD)
{
//command or data?
if (CD == 0) LCDPORT &=~ (1<<LCD_RS); 			//RS = 0 Controlregister
else         LCDPORT |= (1<<LCD_RS); 			//RS = 1 Dataregister
	
LCDPORT = (LCDPORT&0xF0) + ((Data&0xF0)>>4); 	//Write Nibbel MSB
enable_lcd();	
LCDPORT = (LCDPORT&0xF0) + (Data&0x0F); 		//Write Nibbel LSB
enable_lcd();

#ifndef DBG_LCD
_delay_us(50);									//wait till LCD ready again
#endif
}


void put_position_lcd(uint8_t row, uint8_t column)		//4x20 LCD 
{
if      (row <= 1) row = 0x80;	//80 1000 0000
else if (row == 2) row = 0xC0;	//C0 1100 0000
else if (row == 3) row = 0x94;  //90 1001 0100
else 	           row = 0xD4;  //D0 1101 0100
row += column;
write_lcd(row,0);
}


void put_char_lcd(char c)
{
write_lcd(c, 1);	
}


void put_string_p_lcd(const char *Buffer)		
{
uint8_t c;
while ((c = pgm_read_byte(Buffer++)))
    {
    write_lcd(c, 1);
	}
}


void put_string_lcd(char *Buffer)		
{
while (*Buffer != 0)
	{
	write_lcd(*Buffer++,1);
	};
}

void put_msg_lcd(uint8_t msgId)
{
uint8_t i, c;
for (i=0; i<16; i++)
	{
	c= pgm_read_byte(&Msg[msgId][i]);
	write_lcd(c, 1);
	}
write_lcd(' ', 1);
write_lcd(' ', 1);
write_lcd(' ', 1);
write_lcd(' ', 1);
}

void put_number_lcd(int number, char mode)
{
char tmp[3];									//0-999 allowed
if (mode == PLAIN)								//display the number
	{
	tmp[0]= ' ';
	tmp[1]= ' ';
	tmp[2]= ' ';
	itoa(number, tmp, 10);
	put_string_lcd(tmp);
	}
else if (mode == HEX)
    {
	tmp[0]= (number>>4);
	tmp[1]= (number&0x0F);
	if (tmp[0] < 10) write_lcd((tmp[0] +'0'), 1);
	else			 write_lcd((tmp[0] -10 +'A'), 1);
	if (tmp[1] < 10) write_lcd((tmp[1] +'0'), 1);
	else			 write_lcd((tmp[1] -10 +'A'), 1);
	}
else
	{
#ifdef USE_BARGRAPH
	number= (number>>5);
	if (number == 7)
		{
		write_lcd(0xFF,1);
		}
	else 
		{
		write_lcd(number,1);
		}
#endif
	}
}


void clear_lcd(void)
{
write_lcd (1,0); 								//Clear Display
#ifndef DBG_LCD
_delay_ms(5);	
#endif
write_lcd(0x80,0);								//Set DD-Ram Adresse = 0
}


void init_lcd(void)
{	
char tmp = LCD_ROWS;
LCDDDR |= 0b00111111;
#ifndef DBG_LCD
_delay_ms(25);	
_delay_ms(25);
#endif
LCDPORT= 0b00000011;
enable_lcd();
#ifndef DBG_LCD
_delay_ms(5);	
#endif
enable_lcd();
#ifndef DBG_LCD
_delay_ms(5);
#endif
enable_lcd();
#ifndef DBG_LCD
_delay_ms(5);
#endif
LCDPORT= 0b00000010;							//4bit mode
enable_lcd();
#ifndef DBG_LCD
_delay_ms(5);
#endif

#ifdef DOGM
tmp = LCD_ROWS;
if (tmp == 1) write_lcd(0x21,0);				//1 row, table1
else          write_lcd(0x29,0);				//more rows, table 1
write_lcd(0x52,0);								//disable booster, contrast enabled
write_lcd(0x69,0);								//enable follower, amplification
write_lcd(0x74,0);								//set contrast
#endif

tmp = LCD_ROWS;
if (tmp == 1) write_lcd(0x20,0);				//1 row
else          write_lcd(0x28,0);				//more rows



write_lcd(0x0C,0);								//display on
write_lcd(0x04,0);

#ifdef USE_BARGRAPH
write_lcd(0x40,0);								//set CGRAM base address
uint8_t i,k;
for (i=0;i<9;i++)
	{
	write_lcd(0x00,1);
	}	

for (i=0;i<8;i++)
	{
	for (k=8;k>0;k--)
		{
		if (k <=  i+1) write_lcd(0x00,1);
		else		   write_lcd(0x1F,1);
		}
	}
#endif

clear_lcd();
}


//********* Encoder ***********
void init_enc(void)
{
int8_t i= 0;											//init encoder
if (PIND &(1<<ENC_A)) i  = 3;
if (PIND &(1<<ENC_B)) i ^= 1;
enc_last = i;
enc_delta= 0;

if (PIND &(1<<ENC_BTN)) BtnCnt= 0;						//push btn released
else	                BtnCnt= 0xFF;					//btn down (block)
}

void get_enc(void)
{
int8_t  i= 0;											//get encoder state (call with ~1kHz)
int8_t  diff;
if (PIND &(1<<ENC_A)) i  = 3;
if (PIND &(1<<ENC_B)) i ^= 1;
diff= enc_last -i;										// difference new - last
if(diff & 1)
   {
	enc_last= i;										// store new as next last
	enc_delta += (diff & 2) - 1;						// bit 1 = direction (+/-)
   }
}


void check_enc(void)
{
int8_t i;												//evaluate encoder (call with ~100Hz)
i= enc_delta;
enc_delta = i &3;							//www.mikrocontroller.net/articles/Drehgeber
i= (i>>2);
if (i != 0)
   {
	EncVal -= i;
	if      (EncVal > 0xFF00) EncVal= EncMax;			//roll over at limits
	else if (EncVal > EncMax) EncVal= 0;
	gFlags |= (1<<ENC_CHANGED);
   }
   
uint8_t cnt= BtnCnt;
if (PIND &(1<<ENC_BTN))
   {
	BtnCnt= 0;
	if      (cnt > BTN_LONG)   gFlags |= (1<<BTN_LONG_PUSHED);		//long push (for escaping menu)
	else if (cnt > BTN_BOUNCE) gFlags |= (1<<BTN_SHORT_PUSHED);		//short push (as enter)
   }
else if (cnt != 0xFF) BtnCnt +=1;	
}