/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 reception library
* Version		: v0.2
* Last updated	: 25.03.08
* Target		: Transceiver Rev.3.01 [ATmega8515]
* Clock			: 8MHz, 16MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/

#include "globals.h"
#include "lib_rdm_out.h"
#include <string.h>
#include <avr/wdt.h>

// ********************* local definitions *********************

enum {EVAL_RDM, DISC_MSG};				//main flags

typedef union {
   uint16_t u16;
   struct {
          uint8_t u8l;
          uint8_t u8h;
          };
} uint16or8;


typedef union {
   uint64_t u64;
   struct {
          uint8_t u8a;  //LSB
          uint8_t u8b;
		  uint8_t u8c;
		  uint8_t u8d;
		  uint8_t u8e;
		  uint8_t u8f;  //MSB
		  uint8_t u8g; 	//dummy
		  uint8_t u8h; 	//dummy
          };
} uint64or8;

		 
		 uint8_t	 TransNo;								//Tx transaction number counter
		 uint64or8   LoBound;								//lower boundery of discovery range
		 uint64or8	 HiBound;								//higher boundery of discovery range


const uint8_t MsgHead[] PROGMEM = {					   //RDM message header
0xCC, 1, 0x11,												//ASC, SSC, dummy length
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 						//dest UID (broadcast)
0x41, 0x64, 0xFF, 0x00, 0x00, 0x10,							//source UID
0, 1, 0,													//transaction number, port id, msg cnt
0, 0														//sub device
};


// *************** RDM Reception Initialisation ****************
/*
void init_RDM_TX(void)
{
DDRD  |= (1<<PD2)|(1<<PD1);
PORTD |= (1<<PD2)|(1<<PD1);										//enable transmission, IDLE state when tx off
UBRRH  = 0;
UBRRL  = ((F_OSC/4000)-1);									//250kbaud, 8N2
UCSRB  = 0;													//disable USART
UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);

uint8_t i;
for (i=0; i<sizeof(DmxField); i++)
	{
	DmxField[i]= 0;
	}
}
*/


// **** helper functions ****
static inline uint16_t swapInt(uint16_t val)		//change byte order
{
uint16_t res  = (val<<8);
res |= (val>>8);
return (res);
}

uint8_t wait_for_rxc(uint8_t time)
{
SysCnt= 0;											
while (time > SysCnt)
   {
   if (UCSRA &(1<<RXC)) return(0);
   }
UCSRB= 0;												//time out (release USART)
return (2);
}

void wait(uint8_t time)
{
wdt_reset();
SysCnt= 0;											
while (time > SysCnt)
   {
   asm volatile ("nop");
   }
return ;
}


uint16_t build_cs(void)									//build additive check sum
{
uint16_t res= 0;
uint8_t  i;
for (i=0; i<DmxField[2]; i++)
	{
	res += DmxField[i];
	}
return (res);
}




// ****************** RDM data handling *******************
uint8_t DiscRDM(void)									//0= OK; 1= collision; 2= time out
{
// low and high bounds are set globally
uint16or8 cs;
uint8_t   cnt, buf;
while (UCSRB != 0)										//wait for complete DMX transmission
   {
   asm volatile ("nop");
   }
PORTD |= (1<<PD2);										//enable transmission
// complete discovery request packet
memcpy_P(DmxField, MsgHead, sizeof(MsgHead));
struct RDM_Packet *rdm;
rdm = (struct RDM_Packet *)&DmxField;
rdm->Cmd    = DISC_CMD;
rdm->PID    = DISC_UNIQUE_BRANCH;
rdm->PDLen  = 12;
rdm->Length = 12 +24;
rdm->TransNo= TransNo++;								//set transaction number
rdm->Data[0]=  LoBound.u8f;                				//set lower bound
rdm->Data[1]=  LoBound.u8e;
rdm->Data[2]=  LoBound.u8d;
rdm->Data[3]=  LoBound.u8c;
rdm->Data[4]=  LoBound.u8b;
rdm->Data[5]=  LoBound.u8a;
rdm->Data[6]=  HiBound.u8f;                 			//set higher bound
rdm->Data[7]=  HiBound.u8e;
rdm->Data[8]=  HiBound.u8d;
rdm->Data[9]=  HiBound.u8c;
rdm->Data[10]= HiBound.u8b;
rdm->Data[11]= HiBound.u8a;
cs.u16= build_cs();										//build check sum
DmxField[36]= cs.u8h;
DmxField[37]= cs.u8l;
//transmit request
_delay_ms(1);
wdt_reset();
UBRRH  = 0;
UBRRL  = 21;											//45.5kbaud, 8N2
UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
UCSRB  = (1<<TXEN);
UDR    = 0;												//send break
buf= rdm->Length +2;
loop_until_bit_is_set(UCSRA, TXC);						//wait until break complete
UCSRA |= (1<<TXC);
UBRRL  = ((F_OSC/4000)-1);								//250kbaud
for (cnt=0; cnt< buf; cnt++)
   {
   _delay_us(5);
   UDR= DmxField[cnt];
   loop_until_bit_is_set(UCSRA, TXC);
   UCSRA |= (1<<TXC);
   }
_delay_us(5);
//*** receive response
PORTD &= ~(1<<PD2);										//enable reception
UCSRB  = (1<<RXEN);
//eliminate preamble
cnt= 10;
do
   {
   buf= wait_for_rxc(200);									
   if (buf != 0)      return(2);
   buf= UDR;
   if (--cnt == 0)    
      {
	  UCSRB= 0;
	                  return(1);						//collisions may kill preamble
	  }
   }
while (buf != 0xAA);
//get packet
for (cnt=0; cnt<16; cnt++)								
   {
   buf= wait_for_rxc(100);
   if (buf != 0)      return(2);
   DmxField[cnt]= UDR;
   }
UCSRB= 0;												//disable USART
//check range
uint64or8 uid;											
uid.u8h = 0;
uid.u8g = 0;
uid.u8f = DmxField[0]  &DmxField[1];
uid.u8e = DmxField[2]  &DmxField[3];
uid.u8d = DmxField[4]  &DmxField[5];
uid.u8c = DmxField[6]  &DmxField[7];
uid.u8b = DmxField[8]  &DmxField[9];
uid.u8a = DmxField[10] &DmxField[11];

//check cs
cs.u16=0;												
for (cnt=0; cnt<12; cnt++)
   {
   cs.u16 += DmxField[cnt];
   }
buf= DmxField[12] &DmxField[13];
if (buf != cs.u8h) return(1);
buf= DmxField[14] &DmxField[15];
if (buf != cs.u8l) return(1);
buf= NumDev;
ToD[buf][0]= uid.u8f;									//store valid uid in empty slot 
ToD[buf][1]= uid.u8e;
ToD[buf][2]= uid.u8d;
ToD[buf][3]= uid.u8c;
ToD[buf][4]= uid.u8b;
ToD[buf][5]= uid.u8a;
return(0);
}


uint8_t WriteRDM(uint8_t trgt)							//0= OK; 1=frame error; 2= time out; 3= CS error; 4= NACK; 5= invalid dev
{
uint16or8 cs;
uint8_t   cnt, buf, len;
wdt_reset();
while (UCSRB != 0)										//wait for complete DMX transmission
   {
   asm volatile ("nop");
   }
PORTD |= (1<<PD2);										//enable transmission
// complete request packet
memcpy_P(DmxField, MsgHead, sizeof(MsgHead));
struct RDM_Packet *rdm;
rdm = (struct RDM_Packet *)&DmxField;
rdm->Length= rdm->PDLen +24;							//set packet length
if (trgt != 0xFF)
	{
	if (trgt > NumDev) return(5);						//dev doesn't exist
	rdm->DestID[0]= ToD[trgt][0];						//set destination address
	rdm->DestID[1]= ToD[trgt][1];
	rdm->DestID[2]= ToD[trgt][2];
	rdm->DestID[3]= ToD[trgt][3];
	rdm->DestID[4]= ToD[trgt][4];
	rdm->DestID[5]= ToD[trgt][5];
	}
rdm->TransNo= TransNo++;								//set transaction number
cs.u16= build_cs();										//build check sum
DmxField[rdm->Length]= cs.u8h;
DmxField[rdm->Length +1]= cs.u8l;
//transmit request
_delay_ms(1);
wdt_reset();
UBRRH  = 0;
UBRRL  = 21;											//45.5kbaud, 8N2
UCSRC  = (1<<URSEL)|(3<<UCSZ0)|(1<<USBS);
UCSRB  = (1<<TXEN);
UDR    = 0;												//send break
buf= rdm->Length +2;
loop_until_bit_is_set(UCSRA, TXC);						//wait until break complete
UCSRA |= (1<<TXC);
UBRRL  = ((F_OSC/4000)-1);								//250kbaud
for (cnt=0; cnt< buf; cnt++)
   {
   _delay_us(5);
   UDR= DmxField[cnt];
   loop_until_bit_is_set(UCSRA, TXC);
   UCSRA |= (1<<TXC);
   }
for (cnt=0; cnt< 10; cnt++)
   {
   DmxField[cnt]= 0xAA;
   }
_delay_us(4);
//receive response
PORTD &= ~(1<<PD2);										//enable reception
wdt_reset();
UCSRB  = (1<<RXEN);
do
   {
   cnt= wait_for_rxc(200);
   if (cnt != 0)      return(2);						
   buf= UDR;
   }
while (buf != 0xCC);									//wait for ASC
DmxField[0]= buf;						
buf= wait_for_rxc(100);
if (buf != 0)         return(2);
DmxField[1]= UDR;										//SSC
buf= wait_for_rxc(100);
if (buf != 0)         return(2);
len= UDR;
DmxField[2]= len;										//LEN
len += 2;												//+checksum
if (len > sizeof(DmxField)) 
   {
   UCSRB= 0;					  
					  return(1);  						//packet to big for buffer
   }

for (cnt=3; cnt < len; cnt++)
   {
   buf= wait_for_rxc(100);
   if (buf != 0)      return(2);
   DmxField[cnt]= UDR;
   }
UCSRB= 0;												//disable USART
//build checksum
cs.u16= build_cs();
if (rdm->Data[rdm->PDLen]    != cs.u8h) return(3);
if (rdm->Data[(rdm->PDLen +1)] != cs.u8l) return(3);
if (rdm->PortID != RESPONSE_TYPE_ACK)   return(4);      //receiver cannot process PID (but communication was OK)
wdt_reset();
return(0);												//YEAH!!
}


uint8_t rdm_mute(uint8_t trgt)
{
uint8_t res;
struct RDM_Packet *rdm;
rdm = (struct RDM_Packet *)&DmxField;
rdm->Cmd  	 = DISC_CMD;
rdm->PID     = DISC_MUTE;
rdm->PDLen   = 0;
res= WriteRDM(trgt);
return (res);
}


uint8_t rdm_unmute(uint8_t trgt)
{
uint8_t res;
struct RDM_Packet *rdm;
rdm = (struct RDM_Packet *)&DmxField;
rdm->Cmd  	 = DISC_CMD;
rdm->PID     = DISC_UN_MUTE;
rdm->PDLen   = 0;
res= WriteRDM(trgt);
return (res);
}


uint8_t rdm_identify(uint8_t trgt, uint8_t val)
{
uint8_t res;
struct RDM_Packet *rdm;
rdm = (struct RDM_Packet *)&DmxField;
rdm->Cmd  	 = SET_CMD;
rdm->PID     = IDENTIFY;
rdm->PDLen   = 1;
rdm->Data[0] = (val &0b01);
res= WriteRDM(trgt);
return (res);
}


uint8_t rdm_set_address(uint16_t adr)
{
if (adr == 0)  return (4);
if (adr > 512) return (4);
uint8_t res;
struct RDM_Packet *rdm;
rdm = (struct RDM_Packet *)&DmxField;
rdm->Cmd  	 = SET_CMD;
rdm->PID     = DMX_START_ADDRESS;
rdm->PDLen   = 2;
rdm->Data[0] = (adr>>8)&0xFF;
rdm->Data[1] = adr&0xFF;
res= WriteRDM(dev.ID);
return (res);
}


uint8_t rdm_set_personality(uint8_t pers)
{
uint8_t res;
struct RDM_Packet *rdm;
rdm = (struct RDM_Packet *)&DmxField;
rdm->Cmd  	 = SET_CMD;
rdm->PID     = PERSONALITY;
rdm->PDLen   = 1;
rdm->Data[0] = pers;
res= WriteRDM(dev.ID);
return (res);
}


uint8_t rdm_get_devinfo(void)
{
uint16or8 buf;
uint8_t res;
struct RDM_Packet *rdm;
rdm = (struct RDM_Packet *)&DmxField;
rdm->Cmd  	 = GET_CMD;
rdm->PID     = DEVICE_INFO;
rdm->PDLen   = 0;
res= WriteRDM(dev.ID);
if (res==0)
   {
// rdm->Data[2]  | rdm->Data[3]  = device type id (chosen by manufacturer)
// rdm->Data[4]  | rdm->Data[5]  = fixture type id
// rdm->Data[6]  | rdm->Data[7]  | rdm->Data[8]  | rdm->Data[9]  = firmware version
// rdm->Data[10] | rdm->Data[11] = DMX footprint
// rdm->Data[12]				 = current personality
// rdm->Data[13]				 = number of personality
// rdm->Data[14] | rdm->Data[15] = DMX start address
	if (rdm->Data[10] == 0) buf.u8l= rdm->Data[11];
	else					buf.u8l= 0xFF;
	dev.Footprint= buf.u8l;
	dev.Personality=      rdm->Data[12];
	dev.NumPersonalities= rdm->Data[13];
	buf.u8h=              rdm->Data[14];
	buf.u8l=              rdm->Data[15];
	dev.Address= buf.u16;
   }
return (res);
}


uint8_t rdm_get_devlabel(void)
{
uint8_t res;
struct RDM_Packet *rdm;
rdm = (struct RDM_Packet *)&DmxField;
rdm->Cmd  	 = GET_CMD;
rdm->PID     = DEVICE_LABEL;
rdm->PDLen   = 0;
res= WriteRDM(dev.ID);
//process data in main because of LCD...
return (res);
}

// ***** DISCOVERY ******
void rdm_discover(void)								//discover devices
{
uint64_t buf_bound;
uint8_t  res, cnt;
for (cnt=0; cnt<50; cnt++)
   {
   wdt_reset();
   LoBound.u64=  0;
   HiBound.u64=  0xFFFFFFFFFFFE;
   if (DiscRDM() == 2) return;							//all devices found and muted -> done
   buf_bound=    0;
   LoBound.u64=  0x7FFFFFFFFFFF;
   while ((HiBound.u64 -buf_bound)>1)
      { 
	  wait(200);
	  res= DiscRDM();									//0= dev found (and uid stored @NumDev); 1= collision; 2= no response
	  if (res != 2) buf_bound  = LoBound.u64;		
      else          HiBound.u64= LoBound.u64;	
	  LoBound.u64= (buf_bound +HiBound.u64)>>1;
	  if (res == 0)
	     {
		 wait(200);
		 if (rdm_mute(NumDev) == 0)							//found device successfully muted
		    {
			NumDev++;
			if (NumDev >= MAX_DEVS) return;			//buffer is full
			HiBound.u64= 0;								//exit loop and start new search
			buf_bound= 0;
			}
		 }
	  }
   }
}







