#include <util/delay.h>

#define DEBUG_RDM								//use Output for debugging
#define MAX_DEVS	(10)						//size of list of RDM devices
#define MAX_PARAMS	(10)

struct RDM_Packet	{
	uint8_t  ASC;								//alternate start code
	uint8_t  SSC;								//sub start code
	uint8_t  Length;							//total packet size
	uint8_t  DestID[6];							//target address
	uint8_t  SrcID[6];				    		//source address
	uint8_t  TransNo;							//transaction number
	uint8_t  PortID;							//Port ID / Transaction Number
	uint8_t  MsgCount;							//message count
	uint16_t SubDev;							//sub device number (root = 0) 
	uint8_t  Cmd;								//command class
	uint16_t PID;								//parameter ID
	uint8_t  PDLen;								//parameter data length in bytes
	uint8_t  Data[24];							//data byte field
};

struct Dev_Parameters{							//description of current parameter
	uint8_t  ID;
	uint16_t Address;
	uint8_t	 Footprint;
	uint8_t	 Personality;
	uint8_t	 NumPersonalities;
}dev;

         uint8_t	 NumDev;					//number of found RDM devices
         uint8_t	 ToD[MAX_DEVS][6];			//Table of devices
        		 

//extern void    init_RDM_TX(void);
extern void    rdm_discover(void);
extern uint8_t rdm_mute(uint8_t trgt);
extern uint8_t rdm_unmute(uint8_t trgt);
extern uint8_t rdm_identify(uint8_t trgt, uint8_t val);
extern uint8_t rdm_get_devinfo(void);
extern uint8_t rdm_get_devlabel(void);
extern uint8_t rdm_set_address(uint16_t adr);
extern uint8_t rdm_set_personality(uint8_t pers);





// RDM CODES (ANSI E1.20)

//command classes
#define DISC_CMD				(0x10)
#define DISC_CMD_RESPONSE		(0x11)
#define GET_CMD					(0x20)
#define GET_CMD_RESPONSE		(0x21)
#define SET_CMD					(0x30)
#define SET_CMD_RESPONSE		(0x31)

// command class defines ??
#define CC_GET					(0x01)
#define CC_SET					(0x02)
#define CC_GET_SET				(0x03)
#define CC_INVALID				(0xFF)

// data type definitions
#define DS_NOT_DEFINED			(0x00)
#define DS_BIT					(0x01)
#define DS_ASCII				(0x02)
#define DS_BYTE					(0x03)
#define DS_SIGNED_BYTE			(0x04)
#define DS_WORD					(0x05)
#define DS_SIGNED_WORD			(0x06)
#define DS_DWORD				(0x07)
#define DS_SIGNED_DWORD			(0x08)

//response type
#define RESPONSE_TYPE_ACK		(0x00)
#define RESPONSE_TYPE_NACK		(0x02)	//with reason!!

//NACK reasons (MSB is always zero - left out!!)
#define NR_UNKNOWN_PID			(0x00)
#define NR_FORMAT_ERROR			(0x01)
#define NR_HARDWARE_FAULT		(0x02)
#define NR_WRITE_PROTECT		(0x04)
#define NR_UNSUPPORTED_CC		(0x05)
#define NR_DATA_OUT_OF_RANGE	(0x06)
#define NR_BUFFER_FULL			(0x07)
#define NR_PACKET_SIZE_UNSUPPORTED (0x08)
#define NR_NO_SUBDEVICE			(0x09)

//parameter IDs !!bytes are swapped!!
#define DISC_UNIQUE_BRANCH		(0x0100)	//network management
#define DISC_MUTE				(0x0200)
#define DISC_UN_MUTE			(0x0300)
#define IDENTIFY				(0x0010)//(0x1000)
#define RESET					(0x0110)
#define STATUS_MESSAGES			(0x3000)
#define SUPPORTED_PARAMETERS	(0x5000)	//information
#define PARAMETER_DESCRIPTION	(0x5100)
#define DEVICE_INFO				(0x6000)
#define DEVICE_LABEL			(0x8200)
#define MANUFACT_LABEL          (0x8100)
#define DMX_START_ADDRESS		(0xF000)	//DMX setup
#define PERSONALITY				(0xE000)	//device personality

//status types
#define STATUS_NONE				(0x00)		//not in queued msgs
#define STATUS_GET_LAST_MSG		(0x01)
#define STATUS_WARNING			(0x03)
#define STATUS_ERROR			(0x04)

//status msg IDs (error types),(MSB is zero and left out...)
#define STS_CAL_FAIL			(0x01)	//slot label
#define STS_SENS_NOT_FOUND		(0x02)	//"
#define STS_SENS_ALWAYS_ON		(0x03)	//"
#define STS_LAMP_DOUSED			(0x11)
#define STS_LAMP_STRIKE			(0x12)
#define STS_OVERTEMP			(0x21)	//SensorID, Temp
#define STS_UNDERTEMP			(0x22)	//"
#define STS_SENS_OUT_RANGE		(0x23)	//sensorID
#define STS_OVERVOLTAGE_PHASE	(0x31)	//Phase, Voltage
#define STS_UNDERVOLTAGE_PHASE	(0x32)	//"
#define STS_OVERCURRENT			(0x33)	//Phase, Current
#define STS_PHASE_ERROR			(0x36)	//Phase
#define STS_BREAKER_TRIP		(0x42)
#define STS_DIM_FAILURE			(0x44)
#define STS_READY				(0x50)	//slot label
#define STS_NOT_READY			(0x51)	//"
#define STS_LOW_FLUID			(0x52)	//"

//slot labels
#define SD_INTENSITY			(0x0001)
#define SD_INTENSITY_MASTER		(0x0002)
#define SD_PAN					(0x0101)
#define SD_TILT					(0x0102)
#define SD_COLOR_WHEEL			(0x0201)
#define SD_STATIC_GOBO_WHEEL	(0x0301)
#define SD_ROTO_GOBO_WHEEL		(0x0302)
#define SD_PRISM_WHEEL			(0x0303)
#define SD_STROBE				(0x0404)
#define SD_UNDEFINED			(0xFFFF)


//product category
#define FIXT_FIXED				(0x0101)	//not moving fixture
#define FIXT_MOVING_YOKE		(0x0102)	//moving head
#define FIXT_MOVING_MIRROR		(0x0103)	//scanner
#define FIXT_PROJECTOR			(0x0300)	//beamer
#define FIXT_ATMOSPHERIC_FX		(0x0401)	//fogger, hazer
#define FIXT_ATMOSPHERIC_PYRO	(0x0402)	//harr, harr...
#define FIXT_DIMMER				(0x0501)	//dimmer pack
#define FIXT_DIM_FLUORESCENT	(0x0502)	//cfl dimmer
#define FIXT_DIM_LED			(0x0509)	//LED dimmer


