
/**** A P P L I C A T I O N   N O T E   ************************************
*
* Title			: DMX512 E1.20 protocol analyzer
* Version		: v1.2
* Last updated	: 16.06.23
* Target		: Transceiver Rev.3.01
* Clock			: 8MHz
*
* written by hendrik hoelscher, www.hoelscher-hi.de
***************************************************************************
 This program is free software; you can redistribute it and/or 
 modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation; either version2 of 
 the License, or (at your option) any later version. 

 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 General Public License for more details. 

 If you have no copy of the GNU General Public License, write to the 
 Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

 For other license models, please contact the author.

;***************************************************************************/

#include "globals.h"
#include "lib_lcd.h"
#include "lib_dmx.h"
#include "lib_rdm_out.h"
#include <avr/wdt.h>

typedef union {
	uint16_t u16;
	struct {
		uint8_t u8l;
		uint8_t u8h;
	};
} uint16or8;


		uint8_t  MenuLevel;
		uint8_t  MenuRefresh;
		uint8_t  TxChannel;
		uint8_t  Seconds;
		uint8_t	 EEbuf[sizeof(dur)]; 	//mirror of EE content

uint8_t eeDurField[6] EEMEM;		

#define	 EE_CHECK	(8)

enum {DMX_STATISTCS, DMX_OVERVIEW, DMX_RECEIVE, DMX_TRANSMIT, RDM_TRANSMIT};		//modes
	
	
//********* EEPROM magic ***********
uint8_t load_durables(void)								//return 0 if succeeded
{
uint8_t i= -1;
if (eeprom_read_byte(&eeDurField[0]) == EE_CHECK)
	{
	eeprom_read_block(EEbuf,&eeDurField[1],sizeof(dur));
	uint8_t *durField;
	durField= (uint8_t *)&dur;
	for (i=0; i<sizeof(dur); i++)
		{
		durField[i] = EEbuf[i];							//load last vals
		}
	i= 0;
	}
return i;
}

void save_durables(void)								//call periodically to update EE content
{
uint8_t i;
uint8_t *durField;
durField= (uint8_t *)&dur;

for (i=0; i<sizeof(dur); i++)
	{
	if (durField[i] != EEbuf[i])							//update changed content
		{
		EEbuf[i]= durField[i];
		eeprom_write_byte(&eeDurField[i +1], durField[i]);
		}
	}
}	
	
void init_system(void)
{
uint8_t i;

wdt_reset();
wdt_enable(WDTO_2S);							//enable watchdog (Timeout= 2s)
wdt_reset();

DDRC   = 0b00000000;							//use DIPs for Options
PORTC  = 0b11111111;

DDRD   = 0b10000110;							//"Spare" = hi inputs (encoder)
PORTD  = 0b11111010;

TCCR1A = 0;
TCCR1B = (1<<CS11);								//set T1 @clk/8 (1us/tick)
TIMSK  = (1<<OCIE1A);
OCR1A  = 1000; 

if (load_durables() != 0)											//invalid EE data
   {
	dur.Flags= 0;
	dur.Mode= DMX_STATISTCS;
	dur.OvAddress= 0;
	dur.RxAddress= 0;
	eeprom_write_byte(&eeDurField[0], EE_CHECK);					//indicate valid data set
	eeprom_write_block(&dur,&eeDurField[1],sizeof(dur));
   }

init_lcd();
init_enc();
MenuLevel= 0;									//init on highest level
EncVal= dur.Mode;
EncMax= RDM_TRANSMIT;
MenuRefresh= 0;
TxChannel= 1;
NumDev = 0;										//clear list of RDM devices
dev.ID= 0;										//no RDM device selected
gFlags = (1<<ENC_CHANGED);
SysCnt = 100;

put_position_lcd(0,0);							//start screen
put_msg_lcd(1);
put_position_lcd(2,0);
put_msg_lcd(2);

for (i=0; i<100; i++)							//wait 1s
   {
	wdt_reset();
	_delay_ms(10);
   }
}



void eval_dmx_statistics(void)
{
gFlags &= ~(1<<BTN_SHORT_PUSHED);	//no short push
gFlags &= ~(1<<BTN_LONG_PUSHED);	//no long push
if (gFlags &(1<<ENC_CHANGED))
   {
	gFlags &= ~(1<<ENC_CHANGED);
	MenuLevel= 0;					//no levels in this mode
	DmxAddress= 1;
	init_DMX_RX();					//start DMX reception
	MenuRefresh= 50;
	DmxRefresh= 0;	  
   }
if (++MenuRefresh >= 100)			//refresh screen with 1Hz
   {
	MenuRefresh= 0; 
	get_DmxLen();					//get Break and MAB timing
	put_position_lcd(0,0);  
	put_msg_lcd(3);					//display statistics
	put_position_lcd(2,0); 
	put_msg_lcd(8);					//channels
	put_position_lcd(2,10);
	put_number_lcd(DmxSize, PLAIN);
	put_position_lcd(3,0);
	put_msg_lcd(9);					//rate
	put_position_lcd(3,6);
	put_number_lcd(DmxRefresh, PLAIN);
	put_position_lcd(4,0);
	put_msg_lcd(0);					//clear line
	put_position_lcd(4,0);
	put_string_p_lcd(PSTR("BRK:"));
	put_number_lcd(BreakLen, PLAIN);
	put_string_p_lcd(PSTR("us MAB:"));
	put_number_lcd(MabLen, PLAIN);
	put_string_p_lcd(PSTR("us"));
	DmxRefresh= 0;	
	gFlags &= ~(1<<SIGNAL);			//make sure DMX stays connected
   }	
}


void eval_dmx_overview(void)
{
uint8_t ch;
if (gFlags &(1<<BTN_SHORT_PUSHED))
   {
	gFlags &= ~(1<<BTN_SHORT_PUSHED);
	gFlags |=  (1<<ENC_CHANGED);
	EncMax= 7;						// 512/80
	EncVal= dur.OvAddress;
	MenuLevel= 1;
   }
else if (gFlags &(1<<BTN_LONG_PUSHED))
   {
	gFlags &= ~(1<<BTN_LONG_PUSHED);
	gFlags |=  (1<<ENC_CHANGED);
	EncMax= RDM_TRANSMIT;
	EncVal= DMX_OVERVIEW;
	MenuLevel= 0;
   }
if (gFlags &(1<<ENC_CHANGED))
   {
	gFlags &= ~(1<<ENC_CHANGED);
	MenuRefresh= 0;
	init_DMX_RX();					//start DMX reception
	put_position_lcd(0,0);
	put_msg_lcd(4);					//display overview
	put_position_lcd(2,0);
	put_msg_lcd(8);					//display channels
	put_position_lcd(3,0);
	put_msg_lcd(0);					//clear line
	put_position_lcd(4,0);
	put_msg_lcd(0);					//clear line
	if (MenuLevel == 0)
	   {
		put_position_lcd(0,0);
		put_char_lcd('>');
	   }
	else
	   {
		put_position_lcd(2,0);
		put_char_lcd('>');
		dur.OvAddress= EncVal;
	   }
	DmxAddress= (dur.OvAddress*80) +1;
	put_position_lcd(2,10);
	put_number_lcd(DmxAddress, PLAIN);
	put_char_lcd('-');
	put_number_lcd((DmxAddress +79), PLAIN);
   }
if (++MenuRefresh >= 200)
   {
	MenuRefresh= 190;
	put_position_lcd(0,0);
	for (ch=0; ch<20; ch++)
	   {
		put_number_lcd(DmxField[ch], BAR);
	   }
	put_position_lcd(2,0);
	for (ch=20; ch<40; ch++)
	   {
		put_number_lcd(DmxField[ch], BAR);
	   }
	put_position_lcd(3,0);
	for (ch=40; ch<60; ch++)
	   {
		put_number_lcd(DmxField[ch], BAR);
	   }
	put_position_lcd(4,0);
	for (ch=60; ch<80; ch++)
	   {
		put_number_lcd(DmxField[ch], BAR);
	   }
   }
}


void eval_dmx_receive(void)
{
if (gFlags &(1<<BTN_SHORT_PUSHED))
   {
	gFlags &= ~(1<<BTN_SHORT_PUSHED);
	gFlags |=  (1<<ENC_CHANGED);
	EncMax= 511;						
	EncVal= dur.RxAddress;
	MenuLevel= 1;
   }
else if (gFlags &(1<<BTN_LONG_PUSHED))
   {
	gFlags &= ~(1<<BTN_LONG_PUSHED);
	gFlags |=  (1<<ENC_CHANGED);
	EncMax= RDM_TRANSMIT;
	EncVal= DMX_RECEIVE;
	MenuLevel= 0;
   }
if (gFlags &(1<<ENC_CHANGED))
   {
	gFlags &= ~(1<<ENC_CHANGED);
	MenuRefresh= 40;
	init_DMX_RX();					//start DMX reception
	put_position_lcd(0,0);
	put_msg_lcd(5);					//display receive
	put_position_lcd(2,0);
	put_msg_lcd(8);					//display channels
	put_position_lcd(3,0);
	put_msg_lcd(0);					//clear line
	put_position_lcd(4,0);
	put_msg_lcd(0);					//clear line
	if (MenuLevel == 0)
	   {
		put_position_lcd(0,0);
		put_char_lcd('>');
	   }
	else
	   {
		put_position_lcd(2,0);
		put_char_lcd('>');
		dur.RxAddress= EncVal;
	   }
	DmxAddress= dur.RxAddress +1;
	put_position_lcd(2,10);
	put_number_lcd(DmxAddress, PLAIN);
	put_char_lcd('-');
	put_number_lcd((DmxAddress +4), PLAIN);
   }
if (++MenuRefresh >= 50)
   {
	MenuRefresh= 40;
	put_position_lcd(3,0);
	put_msg_lcd(0);					//clear line
	put_position_lcd(4,0);
	put_msg_lcd(0);
	put_position_lcd(3,0);
	put_number_lcd(DmxAddress, PLAIN);
	put_position_lcd(3,4);
	put_number_lcd((DmxAddress +1), PLAIN);
	put_position_lcd(3,8);
	put_number_lcd((DmxAddress +2), PLAIN);
	put_position_lcd(3,12);
	put_number_lcd((DmxAddress +3), PLAIN);
	put_position_lcd(3,16);
	put_number_lcd((DmxAddress +4), PLAIN);
	put_position_lcd(4,0);
	put_number_lcd(DmxField[0], PLAIN);
	put_position_lcd(4,4);
	put_number_lcd(DmxField[1], PLAIN);
	put_position_lcd(4,8);
	put_number_lcd(DmxField[2], PLAIN);
	put_position_lcd(4,12);
	put_number_lcd(DmxField[3], PLAIN);
	put_position_lcd(4,16);
	put_number_lcd(DmxField[4], PLAIN);
   }
}


void eval_dmx_transmit(void)
{
uint8_t ch;
DmxField[0]= 0;								//make sure DMX start code is 0 (for safety)
if (gFlags &(1<<BTN_SHORT_PUSHED))
	{
	gFlags &= ~(1<<BTN_SHORT_PUSHED);
	gFlags |=  (1<<ENC_CHANGED);
	if (MenuLevel == 1)
	   {
		EncMax= 0xFF;							//change channel value
		if (TxChannel > DMX_TX_CHANNELS) TxChannel= DMX_TX_CHANNELS;		//for safety...
		EncVal= DmxField[TxChannel];
		MenuLevel= 2;   
	   }
	else
	   {
		EncMax= DMX_TX_CHANNELS;				//change channel
		EncVal= TxChannel;
		MenuLevel= 1;   
	   }
	}
else if (gFlags &(1<<BTN_LONG_PUSHED))
	{
	gFlags &= ~(1<<BTN_LONG_PUSHED);
	gFlags |=  (1<<ENC_CHANGED);
	if (MenuLevel == 1)
	   {
		EncMax= RDM_TRANSMIT;
		EncVal= DMX_RECEIVE;
		MenuLevel= 0;   
	   }
	else
	   {
		EncMax= DMX_TX_CHANNELS;				//change channel
		EncVal= TxChannel;
		MenuLevel= 1;   
	   }
	}
if (gFlags &(1<<ENC_CHANGED))
	{
	gFlags &= ~(1<<ENC_CHANGED);
	put_position_lcd(0,0);
	put_msg_lcd(6);					//display transmit
	put_position_lcd(2,0);
	put_msg_lcd(0);					//clear line
	if (MenuLevel == 0)				//mark entry to be edited
		{
		init_DMX_TX();	
		put_position_lcd(0,0);
		put_char_lcd('>');
		}
	else if (MenuLevel == 1)
		{
		put_position_lcd(2,3);
		put_char_lcd('>');
		TxChannel= EncVal;
		}
	else 
	    {
		put_position_lcd(2,10);
		put_char_lcd('>');	
		if (TxChannel == 0)
		   {
			for (ch=1; ch<41; ch++)
			   {
				DmxField[ch]= EncVal;				//set value for all ch
			   }   
		   }
		else if (TxChannel <= DMX_TX_CHANNELS)
		   {
			DmxField[TxChannel]= EncVal;			//set value for selected ch
		   }
		}
	put_position_lcd(2,1);							//display ch XXX= val
	put_char_lcd('c');
	put_char_lcd('h');
	put_position_lcd(2,4);
	if (TxChannel == 0)
	   {
		put_char_lcd('A'); 
		put_char_lcd('L');
		put_char_lcd('L');
		put_char_lcd(' ');
		put_char_lcd('=');
		put_position_lcd(2,11);
		put_number_lcd(DmxField[1], PLAIN);  
	   }
	else 
	   {
		put_number_lcd(TxChannel, PLAIN);
	    put_char_lcd(' ');
	    put_char_lcd('=');
	    put_position_lcd(2,11);
	    put_number_lcd(DmxField[TxChannel], PLAIN);
	   }
	 put_position_lcd(3,0);
	 for (ch=1; ch<21; ch++)
	    {
		 put_number_lcd(DmxField[ch], BAR);
	    }
	 put_position_lcd(4,0);
	 for (ch=21; ch<41; ch++)
	    {
		 put_number_lcd(DmxField[ch], BAR);
	    }
	}
new_DMX_Tx();
}


void eval_rdm_control(void)
{
if (gFlags &(1<<BTN_SHORT_PUSHED))
   {
	gFlags &= ~(1<<BTN_SHORT_PUSHED);
	gFlags |=  (1<<ENC_CHANGED);
	if (MenuLevel == 0)							//** change device **
	   {
		if (NumDev == 0) rdm_unmute(0xFF);		//unmute all devices if list is empty (full scan)
		put_position_lcd(2,0);
		put_msg_lcd(10);						//display searching...
		rdm_discover();
		if (NumDev == 0)
		   {
			put_position_lcd(0,0);
			put_msg_lcd(11);					//display no RDM Devices   
		   }
		else
		   {
			EncMax= (NumDev -1);
			EncVal= dev.ID;
			MenuLevel= 1;   
		   }
	   }
	else if (MenuLevel == 1)					//** change address **
	   {
		if (!(PINC &(1<<PC1))) rdm_identify(dev.ID, 0); //stop identify
		EncMax= 511;
		EncVal= (dev.Address -1);								
		MenuLevel= 2;
	   }
	else if (MenuLevel == 2)					//** save addr & change personality **
	   {
		rdm_set_address(dev.Address);			//save new address and switch to pers
		EncMax= (dev.NumPersonalities -1);
		EncVal= (dev.Personality -1);
		MenuLevel= 3;
	   }
	else if (MenuLevel == 3)					//** save personality & go back to device**
	   {
		rdm_set_personality(dev.Personality);	//save new pers
		EncMax= (NumDev -1);
		EncVal= dev.ID;
		MenuLevel= 1;
	   }
   }
else if (gFlags &(1<<BTN_LONG_PUSHED))
   {
	gFlags &= ~(1<<BTN_LONG_PUSHED);
	gFlags |=  (1<<ENC_CHANGED);
	if (MenuLevel == 1)							//change mode
	   {
		if (!(PINC &(1<<PC1))) rdm_identify(dev.ID, 0); //stop identify on all devices
		EncMax= RDM_TRANSMIT;
		EncVal= RDM_TRANSMIT;
		MenuLevel= 0;
	   }
	else if (MenuLevel == 2)					//change device
	   {
		EncMax= (NumDev -1);
		EncVal= dev.ID;
		MenuLevel= 1;
	   }
	else if (MenuLevel == 3)					//change address
	   {
		EncMax= 511;
		EncVal= (dev.Address -1);
		MenuLevel= 2;
	   }
    }
if (gFlags &(1<<ENC_CHANGED))
   {
	gFlags &= ~(1<<ENC_CHANGED);
	put_position_lcd(0,0);
	put_msg_lcd(7);							//display RDM control
	put_position_lcd(2,0);
	put_char_lcd(' ');
	put_position_lcd(3,0);
	put_char_lcd(' ');
	put_position_lcd(4,0);
	put_char_lcd(' ');
	if (MenuLevel == 0)							//***enter mode***
	   {
	   put_position_lcd(0,0);
	   put_char_lcd('>');
	   put_position_lcd(2,0);
	   put_msg_lcd(0);							//clear line
	   put_position_lcd(3,0);
	   put_msg_lcd(0);
	   put_position_lcd(4,0);
	   put_msg_lcd(0);					
	   init_DMX_TX();
	   }
	else if (MenuLevel == 1)						//***change device***
	   {
		rdm_identify(0xFF, 0);						//stop identify on all devices
		dev.ID= EncVal;
		put_position_lcd(2,0);
		put_string_p_lcd(PSTR(">Dev:"));  
		uint8_t tmp;
		if (PINC &(1<<2)) tmp= 1;
		else              tmp= rdm_get_devlabel();	//try to get label from device
		if (tmp == 0)
		   {
			struct RDM_Packet *rdm;					//successful, send label to LCD
			rdm = (struct RDM_Packet *)&DmxField;
			if (rdm->PDLen > 14) rdm->PDLen= 14;	//limit to LCD
			put_char_lcd(' ');
			for (tmp= 0; tmp < rdm->PDLen; tmp++)
			   {
				put_char_lcd(rdm->Data[tmp]);
			   }
		   } 
		else
		   {
			put_char_lcd('0');						//not successful, send UID instead
			put_char_lcd('x');
			put_number_lcd(ToD[dev.ID][0], HEX);
			put_number_lcd(ToD[dev.ID][1], HEX);
			put_number_lcd(ToD[dev.ID][2], HEX);
			put_number_lcd(ToD[dev.ID][3], HEX);
			put_number_lcd(ToD[dev.ID][4], HEX);
			put_number_lcd(ToD[dev.ID][5], HEX);   
		   }    
		rdm_get_devinfo();
		put_position_lcd(3,0);
		put_msg_lcd(8);
		put_position_lcd(3,11);
		put_number_lcd(dev.Address, PLAIN);
		put_char_lcd('-');
		put_number_lcd((dev.Address +dev.Footprint), PLAIN);
		put_position_lcd(4,0);
		put_msg_lcd(12);
		put_position_lcd(4,14);
		put_number_lcd(dev.Personality, PLAIN);
		if (!(PINC &(1<<PC1))) rdm_identify(dev.ID, 1);	//start identify for selected device
	   }
	else if (MenuLevel == 2)					//change address
	   {
	    dev.Address= (EncVal +1);
		put_position_lcd(3,0);
		put_msg_lcd(8);
		put_position_lcd(3,0);
		put_char_lcd('>');
		put_position_lcd(3,11);
		put_number_lcd(dev.Address, PLAIN);
		put_char_lcd('-');
		put_number_lcd((dev.Address +dev.Footprint), PLAIN);
		
	   }
	else if (MenuLevel == 3)					//change personality
	   {
		dev.Personality= (EncVal +1);
		put_position_lcd(4,0);
		put_msg_lcd(12);
		put_position_lcd(4,0);
		put_char_lcd('>');
		put_position_lcd(4,14);
		put_number_lcd(dev.Personality, PLAIN);
	   }
   }
}


void eval_mode(void)
{
uint8_t md= dur.Mode;
if (MenuLevel == 0)				//update mode on highest level
   {
	if (PINC &(1<<PC0))
	   {						//RDM off
		if (EncVal > DMX_TRANSMIT) EncVal= DMX_TRANSMIT;
		EncMax= DMX_TRANSMIT;  
	   }
	else
	   {						//RDM allowed
		if (EncVal > RDM_TRANSMIT) EncVal= RDM_TRANSMIT;
		EncMax= RDM_TRANSMIT;
	   }
	
	md= EncVal;
	dur.Mode= md;   
   }
if      (md == DMX_STATISTCS) eval_dmx_statistics();
else if (md == DMX_OVERVIEW)  eval_dmx_overview();
else if (md == DMX_RECEIVE)   eval_dmx_receive();
else if (md == DMX_TRANSMIT)  eval_dmx_transmit();   
else						  eval_rdm_control();
}


int main(void)
{
cli();
init_system();
sei();
for(;;)
{
if (gFlags &(1<<TICK))
	{
	gFlags &= ~(1<<TICK);			//1kHz
	get_enc();
	
	}
if (SysCnt >= 10)
   {
	SysCnt -= 10;					//100Hz (main loop)
	wdt_reset();
	check_enc();
	eval_mode();
	if (++Seconds >= 200)
	   {
		Seconds= 0;
		save_durables();
	   }
   }
};
}



// ****************** system timer ********************
ISR (TIMER1_COMPA_vect)
{
gFlags |= (1<<TICK);							//called with 1kHz
SysCnt++;
OCR1A += 1000;
}
